@@ -1320,11 +1320,13 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
 
         self.unit_price = self.compute_unit_price()
 
-        self.type = 'line'
         self.amount = self.on_change_with_amount()
 
-    @fields.depends('product', methods=['_get_tax_rule_pattern'])
+    @fields.depends('type', 'product', methods=['_get_tax_rule_pattern'])
     def compute_taxes(self, party):
+        if self.type != 'line':
+            return []
+
         taxes = set()
         pattern = self._get_tax_rule_pattern()
         for tax in self.product.customer_taxes_used:
@@ -1365,13 +1367,15 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
     def on_change_quantity(self):
         self.unit_price = self.compute_unit_price()
 
-    @fields.depends(methods=['on_change_quantity'])
+    @fields.depends(methods=['on_change_quantity', 'on_change_with_amount'])
     def on_change_unit(self):
         self.on_change_quantity()
+        self.amount = self.on_change_with_amount()
 
-    @fields.depends(methods=['on_change_quantity'])
+    @fields.depends(methods=['on_change_quantity', 'on_change_with_amount'])
     def on_change_taxes(self):
         self.on_change_quantity()
+        self.amount = self.on_change_with_amount()
 
     @fields.depends('description')
     def on_change_with_summary(self, name=None):
@@ -1567,8 +1571,9 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
             if invoice_line.type != 'line':
                 continue
             if invoice_line not in skips:
-                quantity += Uom.compute_qty(invoice_line.unit,
-                    invoice_line.quantity, self.unit)
+                quantity += Uom.compute_qty(
+                    invoice_line.unit or self.unit, invoice_line.quantity,
+                    self.unit)
         return quantity
 
     def _get_invoice_line_moves(self):
@@ -1642,8 +1647,9 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
             for invoice_line in self.invoice_lines:
                 if (invoice_line.invoice
                         and invoice_line.invoice.state == 'paid'):
-                    quantity += Uom.compute_qty(invoice_line.unit,
-                        invoice_line.quantity, self.unit)
+                    quantity += Uom.compute_qty(
+                        invoice_line.unit or self.unit, invoice_line.quantity,
+                        self.unit)
             return quantity
 
     def _get_shipped_quantity(self, shipment_type):
@@ -1687,13 +1693,14 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
             moved_quantity *= -1
         invoiced_quantity = 0
         for invoice_line in self.invoice_lines:
-            if (
-                    (not invoice_line.invoice
-                        or invoice_line.invoice.state != 'cancelled')
-                    and self.unit and invoice_line.unit):
-                invoiced_quantity += Uom.compute_qty(
-                    invoice_line.unit, invoice_line.quantity, self.unit,
-                    round=False)
+            if (not invoice_line.invoice
+                    or invoice_line.invoice.state != 'cancelled'):
+                if self.unit:
+                    invoiced_quantity += Uom.compute_qty(
+                        invoice_line.unit or self.unit, invoice_line.quantity,
+                        self.unit, round=False)
+                else:
+                    invoiced_quantity += invoice_line.quantity
         actual_quantity = max(moved_quantity, invoiced_quantity, key=abs)
         if self.unit:
             actual_quantity = self.unit.round(actual_quantity)
